#define BLYNK_PRINT Serial

#include <Servo.h>
#include <Stepper.h>
#include <ESP8266WiFi.h>
#include <BlynkSimpleEsp8266.h>

// You should get Auth Token in the Blynk App.
// Go to the Project Settings (nut icon).
char auth[] = "";

// Your WiFi credentials.
// Set password to "" for open networks.
char ssid[] = "";
char pass[] = "";

// number of steps per one revolution is 2048 ( = 4096 half steps)
#define STEPS  2048
 
// stepper motor control pins
#define IN1   D1   // IN1 is connected to NodeMCU pin D1 (GPIO5)
#define IN2   D2   // IN2 is connected to NodeMCU pin D2 (GPIO4)
#define IN3   D3   // IN3 is connected to NodeMCU pin D3 (GPIO0)
#define IN4   D4   // IN4 is connected to NodeMCU pin D4 (GPIO2)

// lever control pin
#define LEVER_SWITCH D5
#define SERVO_DOOR D6
 
// initialize stepper library
Stepper stepper(STEPS, IN4, IN2, IN3, IN1);

Servo servo;
int state = 0;
const int MAX_SERVO_ANGLE = 100;

BLYNK_WRITE(V1)
{
  if(param.asInt() == 1)
  {
    servo.attach(SERVO_DOOR);
    open_mouth();
    servo.detach();
  }
}

BLYNK_WRITE(V2)
{
  if(param.asInt() == 1)
  {
    servo.attach(SERVO_DOOR);
    for(int i = 0; i < MAX_SERVO_ANGLE; i = i + 1)
    {
      servo.write(i);
      delay(10);
    }
    servo.detach();
  }
}

BLYNK_WRITE(V0)
{
  if(param.asInt() == 1)
  {
    Serial.println("START");
    rotate_stepper();
  
    delay(500);
  
    move_mouth();
    Serial.println("DONE");
  }
}

void move_mouth()
{
  servo.attach(SERVO_DOOR);
  open_mouth();
  servo.detach();
}

void open_mouth()
{
  for(int i = MAX_SERVO_ANGLE; i >= 0; i = i - 1)
  {
    servo.write(i);
    delay(10);
  } 

  delay(5000);
  
  for(int i = 0; i < MAX_SERVO_ANGLE; i = i + 1)
  {
    servo.write(i);
    delay(10);
  }
}

void rotate_stepper()
{
  stepper.step( -100 );
  digitalWrite(IN1, LOW);
  digitalWrite(IN2, LOW);
  digitalWrite(IN3, LOW);
  digitalWrite(IN4, LOW);


  // Move the control arm onto lever sensor
  while(digitalRead(LEVER_SWITCH) == 1)
  {
    stepper.step( -10 );
    digitalWrite(IN1, LOW);
    digitalWrite(IN2, LOW);
    digitalWrite(IN3, LOW);
    digitalWrite(IN4, LOW);
    delay(5);
  }
}

void setup()
{
  pinMode(IN1, OUTPUT);
  pinMode(IN2, OUTPUT);
  pinMode(IN3, OUTPUT);
  pinMode(IN4, OUTPUT);
  pinMode(LEVER_SWITCH, INPUT);
  stepper.setSpeed(10);  // set stepper motor speed to 10 rpm
  
  // Debug console
  Serial.begin(9600);

  Blynk.begin(auth, ssid, pass);
  servo.attach(SERVO_DOOR);
  servo.write(0);
  servo.detach();
}

void loop()
{  
  Blynk.run();
}
